library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

use IEEE.NUMERIC_STD.ALL;

use work.ep9_package.all;

entity Aufgabe3_2 is
  port (
    clock   : in  std_logic;
    
	  DILSW   : in  std_logic_vector(7 downto 0);
    SW      : in  std_logic_vector(7 downto 0);
    LED     : out std_logic_vector(7 downto 0);
	 
    Red     : out std_logic;
    Green   : out std_logic;
    Blue    : out std_logic;
	 
    vSync   : out std_logic;
    hSync   : out std_logic
  );
end Aufgabe3_2;

architecture Behavioral of Aufgabe3_2 is
  component sys_pll
  port (-- Clock in ports
    CLK_IN1           : in     std_logic;
    -- Clock out ports
    CLK_OUT_65        : out    std_logic;
    CLK_OUT_25        : out    std_logic;
    CLK_OUT_20        : out    std_logic;
    -- Status and control signals
    RESET             : in     std_logic;
    LOCKED            : out    std_logic
  );
  end component;
  
  signal s_clk25    : std_logic;
  signal s_clk65    : std_logic;
  signal s_clk20    : std_logic;
  signal s_locked   : std_logic;
  signal s_reset    : std_logic;
  
  signal r_counter  : unsigned(16 downto 0);
  signal r_LED      : unsigned( 7 downto 0);
  
  signal r_hCounter : unsigned(10 downto 0);
  signal r_vCounter : unsigned( 9 downto 0);
  signal r_hSync    : std_logic;
  signal r_vSync    : std_logic;
  signal r_hSyncD   : std_logic;
  signal r_vSyncD   : std_logic;
  
  signal r_red      : std_logic;
  signal r_green    : std_logic;
  signal r_blue     : std_logic;
  
  signal r_posX     : unsigned(10 downto 0);
  signal r_posY     : unsigned( 9 downto 0);
  signal r_dirX     : std_logic;
  signal r_dirY     : std_logic;
begin
  
  i_sys_pll : sys_pll
  port map (-- Clock in ports
    CLK_IN1    => clock,
    -- Clock out ports
    CLK_OUT_65 => s_clk65,
    CLK_OUT_25 => s_clk25,
    CLK_OUT_20 => s_clk20,
    -- Status and control signals
    RESET      => SW(0),
    LOCKED     => s_locked
  );
  
  s_reset <= not s_locked;
  
  Red   <= r_red;
  Green <= r_green;
  Blue  <= r_blue;
  
  vSync <= r_vSyncD;
  hSync <= r_hSyncD;
  
  LED <= std_logic_vector(r_LED);
  
  p_counter: process(s_clk20)
  begin
    if rising_edge(s_clk20) then
      if s_reset = '1' then
        r_counter <= (others => '0');
        r_LED <= (others => '0');
      else
        r_counter <= r_counter + 1;
        if r_counter = c_prescaler - 1 then
          r_counter <= (others => '0');
          
          r_LED <= r_LED + 1;
        end if;
        
      end if;
    end if;
  end process;
  
  p_syncCount: process(s_clk65)
  begin
    if rising_edge(s_clk65) then
      if s_reset = '1' then
        r_hCounter <= to_unsigned(0,11);
        r_vCounter <= to_unsigned(0,10);
        r_hSync <= '1';
        r_vSync <= '1';
      else
        -- Hier bitte die Generierung von hSync und vSync einfgen (Aufgabe 3.2)
      end if;
    end if;
  end process;
  
  
  p_colors: process(s_clk65)
  begin
    if rising_edge(s_clk65) then
      if s_reset = '1' then
        r_red    <= '0';
        r_green  <= '0';
        r_blue   <= '0';
        r_posX   <= to_unsigned(512,11);
        r_posY   <= to_unsigned(384,10);
        r_dirX   <= '1';
        r_dirY   <= '1';
        r_hSyncD <= '1';
        r_vSyncD <= '1';
      else
        r_red   <= SW(7);
        r_green <= SW(6);
        r_blue  <= SW(5);
        r_hSyncD <= r_hSync;
        r_vSyncD <= r_vSync;
        
        -- Ab hier die Lsung fr die Aufgaben 3.3 und 3.4 einfgen:
        
        
		  
		  
        -- Diese Zeilen mssen am Ende des Prozesses stehen bleiben!
        if getLine(r_vCounter) < 0 or getLine(r_vCounter) >= c_activeLinesPerFrame then
          r_red   <= '0';
          r_green <= '0';
          r_blue  <= '0';
        end if;
        
        if getPixel(r_hCounter) < 0 or getPixel(r_hCounter)>= c_activePixelPerLine then
          r_red   <= '0';
          r_green <= '0';
          r_blue  <= '0';
        end if;
        
	    end if;
	  end if;
  end process;
  
end Behavioral;

